use anyhow::{anyhow, Result};

/// Get field package path and identifier name from the path.
/// (TODO: Needs to be a package related to the language specification
/// and move this function into it.)
///
/// split_field_path("pkg.to.path:field") -> ("pkg.to.path", "field")
pub(crate) fn split_field_path(path: &str) -> Result<(String, String)> {
    let err = Err(anyhow!("Invalid field path {:?}", path));
    let paths = path.splitn(2, ':').collect::<Vec<&str>>();
    let (pkgpath, field_path) = if paths.len() == 1 {
        ("".to_string(), paths[0].to_string())
    } else if paths.len() == 2 {
        (paths[0].to_string(), paths[1].to_string())
    } else {
        return err;
    };
    if field_path.is_empty() {
        err
    } else {
        Ok((pkgpath, field_path))
    }
}

/// Get the invalid spec error message.
#[inline]
pub(crate) fn invalid_spec_error(spec: &str) -> anyhow::Error {
    anyhow!("Invalid spec format '{}', expected <pkgpath>:<field_path>=<filed_value> or <pkgpath>:<field_path>-", spec)
}
